@@ -1,6 +1,6 @@
 /*  This file is part of the KDE project.
 
-Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
+Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 
 This library is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -57,6 +57,8 @@ namespace Phonon
 
         BackendNode::~BackendNode()
         {
+            //this will remove the filter from the graph
+            mediaObjectDestroyed();
         }
 
         void BackendNode::setMediaObject(MediaObject *mo)
