@@ -1,6 +1,6 @@
 /*  This file is part of the KDE project.
 
-Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
+Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 
 This library is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -36,15 +36,7 @@ namespace Phonon
         //these mediatypes define a stream, its type will be autodetected by DirectShow
         static QVector<AM_MEDIA_TYPE> getMediaTypes()
         {
-            AM_MEDIA_TYPE mt;
-            mt.majortype = MEDIATYPE_Stream;
-            mt.bFixedSizeSamples = TRUE;
-            mt.bTemporalCompression = FALSE;
-            mt.lSampleSize = 1;
-            mt.formattype = GUID_NULL;
-            mt.pUnk = 0;
-            mt.cbFormat = 0;
-            mt.pbFormat = 0;
+            AM_MEDIA_TYPE mt = { MEDIATYPE_Stream, MEDIASUBTYPE_NULL, TRUE, FALSE, 1, GUID_NULL, 0, 0, 0};
 
             QVector<AM_MEDIA_TYPE> ret;
             //normal auto-detect stream
@@ -147,9 +139,6 @@ namespace Phonon
               {
                   QMutexLocker locker(&m_mutexRead);
 
-                  if (m_mediaGraph->isStopping()) {
-                      return VFW_E_WRONG_STATE;
-                  }
 
                   if(streamSize() != 1 && pos + length > streamSize()) {
                       //it tries to read outside of the boundaries
@@ -166,9 +155,6 @@ namespace Phonon
                   int oldSize = currentBufferSize();
                   while (currentBufferSize() < int(length)) {
                       needData();
-                      if (m_mediaGraph->isStopping()) {
-                          return VFW_E_WRONG_STATE;
-                      }
 
                       if (oldSize == currentBufferSize()) {
                           break; //we didn't get any data
