@@ -37,7 +37,7 @@ along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QtCore/QDebug>
 
-#define TIMER_INTERVAL 16 //... ms for the timer that polls the current state (we use the multimedia timer
+#define TIMER_INTERVAL 16 //... ms for the timer that polls the current state (we use the multimedia timer)
 #define PRELOAD_TIME 2000 // 2 seconds to load a source
 
 QT_BEGIN_NAMESPACE
@@ -90,7 +90,7 @@ namespace Phonon
                 DWORD result = ::WaitForMultipleObjects(count, handles, FALSE, INFINITE);
                 if (result == WAIT_OBJECT_0) {
                     if (m_finished) {
-                        //that's the end if the thread execution
+                        //that's the end of the thread execution
                         return;
                     }
 
@@ -208,10 +208,10 @@ namespace Phonon
 
             HRESULT hr = S_OK;
 
+            QMutexLocker locker(&m_mutex);
             m_currentRender = w.graph;
 			m_currentRenderId = w.id;
             if (w.task == ReplaceGraph) {
-                QMutexLocker locker(&m_mutex);
                 HANDLE h;
 
                 int index = -1;
@@ -251,7 +251,7 @@ namespace Phonon
                     hr = w.graph->RenderFile(reinterpret_cast<const wchar_t *>(w.url.utf16()), 0);
                 }
                 if (hr != E_ABORT) {
-					emit asyncRenderFinished(w.id, hr, w.graph);
+                    emit asyncRenderFinished(w.id, hr, w.graph);
                 }
             } else if (w.task == Seek) {
                 //that's a seekrequest
@@ -267,7 +267,7 @@ namespace Phonon
                     }
                 }
                 emit asyncSeekingFinished(w.id, currentTime);
-                hr = E_ABORT; //to avoid emitting asyncRenderFinished 
+                hr = E_ABORT; //to avoid emitting asyncRenderFinished
             } else if (w.task == ChangeState) {
 
                 //remove useless decoders
@@ -343,10 +343,10 @@ namespace Phonon
                 }
             }
 
-			if (m_currentRender && m_currentRenderId == renderId) {
-				m_currentRender->Abort();
-			}
-		}
+            if (m_currentRender && m_currentRenderId == renderId) {
+                m_currentRender->Abort();
+            }
+        }
 
         //tells the thread to stop processing
         void WorkerThread::signalStop()
@@ -384,13 +384,13 @@ namespace Phonon
         {
 
             for(int i = 0; i < FILTER_COUNT; ++i) {
-                m_graphs[i] = new MediaGraph(this, i);                
+                m_graphs[i] = new MediaGraph(this, i);
             }
 
-            connect(&m_thread, SIGNAL(stateReady(Graph, Phonon::State)), 
+            connect(&m_thread, SIGNAL(stateReady(Graph, Phonon::State)),
                                SLOT(slotStateReady(Graph, Phonon::State)));
 
-            connect(&m_thread, SIGNAL(eventReady(Graph, long, long)), 
+            connect(&m_thread, SIGNAL(eventReady(Graph, long, long)),
                                SLOT(handleEvents(Graph, long, long)));
 
             connect(&m_thread, SIGNAL(asyncRenderFinished(quint16, HRESULT, Graph)),
@@ -485,7 +485,7 @@ namespace Phonon
                     }
 
                     if (!m_aboutToFinishSent && remaining < PRELOAD_TIME - m_transitionTime  + TIMER_INTERVAL/2) {
-                        //let's take a 2 seconds time time to actually load the next file
+                        //let's take a 2 seconds time to actually load the next file
 #ifdef GRAPH_DEBUG
                         qDebug() << "DS9: emit aboutToFinish" << remaining << QTime::currentTime().toString();
 #endif
@@ -520,6 +520,18 @@ namespace Phonon
 
             qSwap(m_graphs[0], m_graphs[1]); //swap the graphs
 
+            if (m_transitionTime >= 0)
+                m_graphs[1]->stop(); //make sure we stop the previous graph
+
+            if (currentGraph()->mediaSource().type() != Phonon::MediaSource::Invalid &&
+                catchComError(currentGraph()->renderResult())) {
+                    setState(Phonon::ErrorState);
+                    return;
+            }
+
+            //we need to play the next media
+            play();
+
             //we tell the video widgets to switch now to the new source
 #ifndef QT_NO_PHONON_VIDEO
             for (int i = 0; i < m_videoWidgets.count(); ++i) {
@@ -528,15 +540,6 @@ namespace Phonon
 #endif //QT_NO_PHONON_VIDEO
 
             emit currentSourceChanged(currentGraph()->mediaSource());
-
-            if (currentGraph()->isLoading()) {
-                //will simply tell that when loading is finished 
-                //it should start the playback
-                play(); 
-            }
-
-
-
             emit metaDataChanged(currentGraph()->metadata());
 
             if (nextGraph()->hasVideo() != currentGraph()->hasVideo()) {
@@ -549,15 +552,6 @@ namespace Phonon
 #ifndef QT_NO_PHONON_MEDIACONTROLLER
             setTitles(currentGraph()->titles());
 #endif //QT_NO_PHONON_MEDIACONTROLLER
-
-            //this manages only gapless transitions
-            if (currentGraph()->mediaSource().type() != Phonon::MediaSource::Invalid) {
-                if (catchComError(currentGraph()->renderResult())) {
-                    setState(Phonon::ErrorState);
-                } else {
-                    play();
-                }
-            }
         }
 
         Phonon::State MediaObject::state() const
@@ -583,7 +577,7 @@ namespace Phonon
         {
 #ifndef QT_NO_PHONON_MEDIACONTROLLER
             //1st, check if there is more titles after
-            const qint64 ret = (m_currentTitle < _iface_availableTitles() - 1) ? 
+            const qint64 ret = (m_currentTitle < _iface_availableTitles() - 1) ?
                 titleAbsolutePosition(m_currentTitle+1) : currentGraph()->absoluteTotalTime();
 
             //this is the duration of the current title
@@ -596,7 +590,7 @@ namespace Phonon
         qint64 MediaObject::currentTime() const
         {
             //this handles inaccuracy when stopping on a title
-            return currentGraph()->absoluteCurrentTime() 
+            return currentGraph()->absoluteCurrentTime()
 #ifndef QT_NO_PHONON_MEDIACONTROLLER
                 - titleAbsolutePosition(m_currentTitle)
 #endif //QT_NO_PHONON_MEDIACONTROLLER
@@ -746,7 +740,7 @@ namespace Phonon
             m_oldHasVideo = currentGraph()->hasVideo();
             setState(Phonon::LoadingState);
             //After loading we go into stopped state
-            m_nextState = Phonon::StoppedState; 
+            m_nextState = Phonon::StoppedState;
             catchComError(currentGraph()->loadSource(source));
             emit currentSourceChanged(source);
         }
@@ -760,7 +754,7 @@ namespace Phonon
 
         void MediaObject::loadingFinished(MediaGraph *mg)
         {
-            if (mg == currentGraph()) { 
+            if (mg == currentGraph()) {
 #ifndef QT_NO_PHONON_MEDIACONTROLLER
                 //Title interface
                 m_currentTitle = 0;
@@ -817,7 +811,7 @@ namespace Phonon
 
         void MediaObject::seekingFinished(MediaGraph *mg)
         {
-            if (mg == currentGraph()) { 
+            if (mg == currentGraph()) {
 
                 updateTargetTick();
                 if (currentTime() < totalTime() - m_prefinishMark) {
@@ -892,7 +886,7 @@ namespace Phonon
 #ifndef QT_NO_PHONON_VIDEO
                 if (VideoWidget *video = qobject_cast<VideoWidget*>(sink)) {
                     m_videoWidgets += video;
-                } else 
+                } else
 #endif //QT_NO_PHONON_VIDEO
                     if (AudioOutput *audio = qobject_cast<AudioOutput*>(sink)) {
                     m_audioOutputs += audio;
@@ -911,7 +905,7 @@ namespace Phonon
 #ifndef QT_NO_PHONON_VIDEO
                 if (VideoWidget *video = qobject_cast<VideoWidget*>(sink)) {
                     m_videoWidgets.removeOne(video);
-                } else 
+                } else
 #endif //QT_NO_PHONON_VIDEO
                     if (AudioOutput *audio = qobject_cast<AudioOutput*>(sink)) {
                         m_audioOutputs.removeOne(audio);
@@ -993,7 +987,7 @@ namespace Phonon
                     emit stateChanged(state(), m_state);
                 }
                 break;
-            case EC_LENGTH_CHANGED: 
+            case EC_LENGTH_CHANGED:
                 if (graph == currentGraph()->graph()) {
                     emit totalTimeChanged( totalTime() );
                 }
