@@ -24,7 +24,7 @@
 #include "widgetrenderer.h"
 #include "x11renderer.h"
 #include "artssink.h"
-#include "pulsesupport_p.h"
+#include "pulsesupport.h"
 
 #ifdef USE_ALSASINK2
 #include "alsasink2.h"
@@ -88,7 +88,7 @@ DeviceManager::DeviceManager(Backend *backend)
             m_audioSink = "pulsesink";
     }
     if ("pulsesink" != m_audioSink)
-        pulse->disable();
+        pulse->enable(false);
 
     m_videoSinkWidget = qgetenv("PHONON_GST_VIDEOMODE");
     if (m_videoSinkWidget.isEmpty()) {
@@ -253,6 +253,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
     return sink;
 }
 
+#ifndef QT_NO_PHONON_VIDEO
 AbstractRenderer *DeviceManager::createVideoRenderer(VideoWidget *parent)
 {
 #if !defined(QT_NO_OPENGL) && !defined(QT_OPENGL_ES)
@@ -275,6 +276,7 @@ AbstractRenderer *DeviceManager::createVideoRenderer(VideoWidget *parent)
 #endif
     return new WidgetRenderer(parent);
 }
+#endif //QT_NO_PHONON_VIDEO
 
 /**
  * Allocate a device id for a new audio device
