@@ -39,7 +39,7 @@ namespace Phonon
 class MediaObject;
 class AbstractMediaStreamPrivate;
 
-/** \class AbstractMediaStream abstractmediastream.h Phonon/AbstractMediaStream
+/** \class AbstractMediaStream abstractmediastream.h phonon/AbstractMediaStream
  * \brief Base class for custom media data streams.
  *
  * Implement this class to provide a custom data stream to the backend. The class supports both, the
@@ -214,7 +214,7 @@ class PHONON_EXPORT AbstractMediaStream : public QObject
         virtual void seekStream(qint64 offset);
 
         AbstractMediaStream(AbstractMediaStreamPrivate &dd, QObject *parent);
-        AbstractMediaStreamPrivate *d_ptr;
+        QScopedPointer<AbstractMediaStreamPrivate> d_ptr;
 };
 
 } // namespace Phonon
