@@ -59,6 +59,7 @@ class AudioOutputPrivate : public AbstractAudioOutputPrivate
 #endif
             deviceBeforeFallback(-1),
             outputDeviceOverridden(false),
+            forceMove(false),
             muted(false)
         {
         }
@@ -67,14 +68,17 @@ class AudioOutputPrivate : public AbstractAudioOutputPrivate
 
         enum DeviceChangeType {
             FallbackChange,
-            HigherPreferenceChange
+            HigherPreferenceChange,
+            SoundSystemChange
         };
         void handleAutomaticDeviceChange(const AudioOutputDevice &newDev, DeviceChangeType type);
 
         void _k_volumeChanged(qreal);
+        void _k_mutedChanged(bool);
         void _k_revertFallback();
         void _k_audioDeviceFailed();
         void _k_deviceListChanged();
+        void _k_deviceChanged(int deviceIndex);
 
     private:
         QString name;
@@ -87,6 +91,7 @@ class AudioOutputPrivate : public AbstractAudioOutputPrivate
         Category category;
         int deviceBeforeFallback;
         bool outputDeviceOverridden;
+        bool forceMove;
         bool muted;
 };
 } //namespace Phonon
