@@ -20,20 +20,23 @@
 
 */
 
-#include <QtCore/QtGlobal>
-#include <QtCore/QEventLoop>
+#include <QtCore/QAbstractEventDispatcher>
 #include <QtCore/QDebug>
 #include <QtCore/QStringList>
+#include <QTimer>
 
 #ifdef HAVE_PULSEAUDIO
+#include "pulsestream_p.h"
 #include <glib.h>
 #include <pulse/pulseaudio.h>
 #include <pulse/xmalloc.h>
 #include <pulse/glib-mainloop.h>
-#include <pulse/ext-device-manager.h>
+#ifdef HAVE_PULSEAUDIO_DEVICE_MANAGER
+#  include <pulse/ext-device-manager.h>
+#endif
 #endif // HAVE_PULSEAUDIO
 
-#include "pulsesupport_p.h"
+#include "pulsesupport.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -133,7 +136,6 @@ static bool s_pulseActive = false;
 
 static pa_glib_mainloop *s_mainloop = NULL;
 static pa_context *s_context = NULL;
-static QEventLoop *s_connectionEventloop = NULL;
 
 
 
@@ -142,63 +144,70 @@ static int s_deviceIndexCounter = 0;
 static QMap<QString, int> s_outputDeviceIndexes;
 static QMap<int, AudioDevice> s_outputDevices;
 static QMap<Phonon::Category, QMap<int, int> > s_outputDevicePriorities; // prio, device
-static QMap<QString, uint32_t> s_outputStreamIndexMap;
-static QMap<QString, int> s_outputStreamMoveQueue;
+static QMap<QString, PulseStream*> s_outputStreams;
 
 static QMap<QString, int> s_captureDeviceIndexes;
 static QMap<int, AudioDevice> s_captureDevices;
 static QMap<Phonon::Category, QMap<int, int> > s_captureDevicePriorities; // prio, device
-static QMap<QString, uint32_t> s_captureStreamIndexMap;
-static QMap<QString, int> s_captureStreamMoveQueue;
+static QMap<QString, PulseStream*> s_captureStreams;
 
+static PulseStream* findStreamByPulseIndex(QMap<QString, PulseStream*> map, uint32_t index)
+{
+  QMap<QString, PulseStream*>::iterator it;
+  for (it = map.begin(); it != map.end(); ++it)
+    if ((*it)->index() == index)
+      return *it;
+  return NULL;
+}
 
-static void ext_device_manager_subscribe_cb(pa_context *, void *);
+static void createGenericDevices()
+{
+    // OK so we don't have the device manager extension, but we can show a single device and fake it.
+    int index;
+    s_outputDeviceIndexes.clear();
+    s_outputDevices.clear();
+    s_outputDevicePriorities.clear();
+    index = s_deviceIndexCounter++;
+    s_outputDeviceIndexes.insert("sink:default", index);
+    s_outputDevices.insert(index, AudioDevice("sink:default", QObject::tr("PulseAudio Sound Server"), "audio-backend-pulseaudio", 0));
+    for (int i = Phonon::NoCategory; i <= Phonon::LastCategory; ++i) {
+        Phonon::Category cat = static_cast<Phonon::Category>(i);
+        s_outputDevicePriorities[cat].insert(0, index);
+    }
+
+    s_captureDeviceIndexes.clear();
+    s_captureDevices.clear();
+    s_captureDevicePriorities.clear();
+    index = s_deviceIndexCounter++;
+    s_captureDeviceIndexes.insert("source:default", index);
+    s_captureDevices.insert(index, AudioDevice("source:default", QObject::tr("PulseAudio Sound Server"), "audio-backend-pulseaudio", 0));
+    for (int i = Phonon::NoCategory; i <= Phonon::LastCategory; ++i) {
+        Phonon::Category cat = static_cast<Phonon::Category>(i);
+        s_captureDevicePriorities[cat].insert(0, index);
+    }
+}
+
+#ifdef HAVE_PULSEAUDIO_DEVICE_MANAGER
 static void ext_device_manager_read_cb(pa_context *c, const pa_ext_device_manager_info *info, int eol, void *userdata) {
     Q_ASSERT(c);
     Q_ASSERT(userdata);
 
-    // If this is our first iteration, set things up properly
-    if (s_connectionEventloop) {
-        s_connectionEventloop->exit(0);
-        s_connectionEventloop = NULL;
-        s_pulseActive = true;
-
-        pa_operation *o;
-        pa_ext_device_manager_set_subscribe_cb(c, ext_device_manager_subscribe_cb, NULL);
-        if ((o = pa_ext_device_manager_subscribe(c, 1, NULL, NULL)))
-            pa_operation_unref(o);
-    }
+    PulseUserData *u = reinterpret_cast<PulseUserData*>(userdata);
 
     if (eol < 0) {
         logMessage(QString("Failed to initialize device manager extension: %1").arg(pa_strerror(pa_context_errno(c))));
-        // OK so we don't have the device manager extension, but we can show a single device and fake it.
-        int index;
-        s_outputDeviceIndexes.clear();
-        s_outputDevices.clear();
-        s_outputDevicePriorities.clear();
-        index = s_deviceIndexCounter++;
-        s_outputDeviceIndexes.insert("sink:default", index);
-        s_outputDevices.insert(index, AudioDevice("sink:default", QObject::tr("PulseAudio Sound Server").toUtf8(), "audio-backend-pulseaudio", 0));
-        for (int i = Phonon::NoCategory; i <= Phonon::LastCategory; ++i) {
-            Phonon::Category cat = static_cast<Phonon::Category>(i);
-            s_outputDevicePriorities[cat].insert(0, index);
-        }
-
-        s_captureDeviceIndexes.clear();
-        s_captureDevices.clear();
-        s_captureDevicePriorities.clear();
-        index = s_deviceIndexCounter++;
-        s_captureDeviceIndexes.insert("source:default", index);
-        s_captureDevices.insert(index, AudioDevice("source:default", QObject::tr("PulseAudio Sound Server").toUtf8(), "audio-backend-pulseaudio", 0));
-        for (int i = Phonon::NoCategory; i <= Phonon::LastCategory; ++i) {
-            Phonon::Category cat = static_cast<Phonon::Category>(i);
-            s_captureDevicePriorities[cat].insert(0, index);
+        if (s_context != c) {
+            logMessage("Falling back to single device mode");
+            // Only create our gerneric devices during the probe phase.
+            createGenericDevices();
+            // As this is our probe phase, exit immediately
+            pa_context_disconnect(c);
         }
+        delete u;
 
         return;
     }
 
-    PulseUserData *u = reinterpret_cast<PulseUserData*>(userdata);
     if (eol) {
         // We're done reading the data, so order it by priority and copy it into the
         // static variables where it can then be accessed by those classes that need it.
@@ -284,6 +293,8 @@ static void ext_device_manager_read_cb(pa_context *c, const pa_ext_device_manage
         }
 
         if (s_instance) {
+            // This wont be emitted durring the connection probe phase
+            // which is intensional
             if (output_changed)
                 s_instance->emitObjectDescriptionChanged(AudioOutputDeviceType);
             if (capture_changed)
@@ -318,6 +329,11 @@ static void ext_device_manager_read_cb(pa_context *c, const pa_ext_device_manage
                 }
             }
         }
+
+        // If this is our probe phase, exit now as we're finished reading
+        // our device info and can exit and reconnect
+        if (s_context != c)
+            pa_context_disconnect(c);
     }
 
     if (!info)
@@ -355,7 +371,7 @@ static void ext_device_manager_read_cb(pa_context *c, const pa_ext_device_manage
     }
 
     // Add the new device itself.
-    new_devices->insert(name, AudioDevice(name, info->description, info->icon, info->index));
+    new_devices->insert(name, AudioDevice(name, QString::fromUtf8(info->description), QString::fromUtf8(info->icon), info->index));
 
     // For each role in the priority, map it to a phonon category and store the order.
     for (uint32_t i = 0; i < info->n_role_priorities; ++i) {
@@ -370,79 +386,19 @@ static void ext_device_manager_read_cb(pa_context *c, const pa_ext_device_manage
     }
 }
 
-static void set_output_device(QString streamUuid)
-{
-    // If we only have one device, bail. This will be true if we are not using module-device-manager
-    if (s_outputDevices.size() < 2)
-        return;
+static void ext_device_manager_subscribe_cb(pa_context *c, void *) {
+    Q_ASSERT(c);
 
-    if (!s_outputStreamMoveQueue.contains(streamUuid))
-        return;
-
-    if (!s_outputStreamIndexMap.contains(streamUuid))
-        return;
-
-    if (s_outputStreamIndexMap[streamUuid] == PA_INVALID_INDEX)
-        return;
-
-    int device = s_outputStreamMoveQueue[streamUuid];
-    if (!s_outputDevices.contains(device))
-        return;
-
-    // We don't remove the uuid from the s_captureStreamMoveQueue
-    // as an application may reuse the phonon AudioOutput object
-
-    uint32_t pulse_device_index = s_outputDevices[device].pulseIndex;
-    uint32_t pulse_stream_index = s_outputStreamIndexMap[streamUuid];
-
-    const QVariant var = s_outputDevices[device].properties["name"];
-    logMessage(QString("Moving Pulse Sink Input %1 to '%2' (Pulse Sink %3)").arg(pulse_stream_index).arg(var.toString()).arg(pulse_device_index));
-
-    /// @todo Find a way to move the stream without saving it... We don't want to pollute the stream restore db.
-    pa_operation* o;
-    if (!(o = pa_context_move_sink_input_by_index(s_context, pulse_stream_index, pulse_device_index, NULL, NULL))) {
-        logMessage(QString("pa_context_move_sink_input_by_index() failed"));
-        return;
-    }
-    pa_operation_unref(o);
-}
-
-static void set_capture_device(QString streamUuid)
-{
-    // If we only have one device, bail. This will be true if we are not using module-device-manager
-    if (s_captureDevices.size() < 2)
-        return;
-
-    if (!s_captureStreamMoveQueue.contains(streamUuid))
-        return;
-
-    if (!s_captureStreamIndexMap.contains(streamUuid))
-        return;
-
-    if (s_captureStreamIndexMap[streamUuid] == PA_INVALID_INDEX)
-        return;
-
-    int device = s_captureStreamMoveQueue[streamUuid];
-    if (!s_captureDevices.contains(device))
-        return;
-
-    // We don't remove the uuid from the s_captureStreamMoveQueue
-    // as an application may reuse the phonon AudioCapture object (when it exists!)
-
-    uint32_t pulse_device_index = s_captureDevices[device].pulseIndex;
-    uint32_t pulse_stream_index = s_captureStreamIndexMap[streamUuid];
-
-    const QVariant var = s_captureDevices[device].properties["name"];
-    logMessage(QString("Moving Pulse Source Output %1 to '%2' (Pulse Sink %3)").arg(pulse_stream_index).arg(var.toString()).arg(pulse_device_index));
-
-    /// @todo Find a way to move the stream without saving it... We don't want to pollute the stream restore db.
-    pa_operation* o;
-    if (!(o = pa_context_move_source_output_by_index(s_context, pulse_stream_index, pulse_device_index, NULL, NULL))) {
-        logMessage(QString("pa_context_move_source_output_by_index() failed"));
+    pa_operation *o;
+    PulseUserData *u = new PulseUserData;
+    if (!(o = pa_ext_device_manager_read(c, ext_device_manager_read_cb, u))) {
+        logMessage(QString("pa_ext_device_manager_read() failed."));
+        delete u;
         return;
     }
     pa_operation_unref(o);
 }
+#endif
 
 void sink_input_cb(pa_context *c, const pa_sink_input_info *i, int eol, void *userdata) {
     Q_UNUSED(userdata);
@@ -465,9 +421,25 @@ void sink_input_cb(pa_context *c, const pa_sink_input_info *i, int eol, void *us
     const char *t;
     if ((t = pa_proplist_gets(i->proplist, "phonon.streamid"))) {
         logMessage(QString("Found PulseAudio stream index %1 for Phonon Output Stream %2").arg(i->index).arg(t));
-        s_outputStreamIndexMap[QString(t)] = i->index;
-        // Process any pending moves...
-        set_output_device(QString(t));
+
+        // We only care about our own streams (other phonon processes are irrelevent)
+        if (s_outputStreams.contains(QString(t))) {
+            PulseStream *stream = s_outputStreams[QString(t)];
+            stream->setIndex(i->index);
+            stream->setVolume(&i->volume);
+            stream->setMute(!!i->mute);
+
+            // Find the sink's phonon index and notify whoever cares...
+            if (PA_INVALID_INDEX != i->sink) {
+                QMap<int, AudioDevice>::iterator it;
+                for (it = s_outputDevices.begin(); it != s_outputDevices.end(); ++it) {
+                    if ((*it).pulseIndex == i->sink) {
+                        stream->setDevice(it.key());
+                        break;
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -492,9 +464,25 @@ void source_output_cb(pa_context *c, const pa_source_output_info *i, int eol, vo
     const char *t;
     if ((t = pa_proplist_gets(i->proplist, "phonon.streamid"))) {
         logMessage(QString("Found PulseAudio stream index %1 for Phonon Capture Stream %2").arg(i->index).arg(t));
-        s_captureStreamIndexMap[QString(t)] = i->index;
-        // Process any pending moves...
-        set_capture_device(QString(t));
+
+        // We only care about our own streams (other phonon processes are irrelevent)
+        if (s_captureStreams.contains(QString(t))) {
+            PulseStream *stream = s_captureStreams[QString(t)];
+            stream->setIndex(i->index);
+            //stream->setVolume(&i->volume);
+            //stream->setMute(!!i->mute);
+
+            // Find the source's phonon index and notify whoever cares...
+            if (PA_INVALID_INDEX != i->source) {
+                QMap<int, AudioDevice>::iterator it;
+                for (it = s_captureDevices.begin(); it != s_captureDevices.end(); ++it) {
+                    if ((*it).pulseIndex == i->source) {
+                        stream->setDevice(it.key());
+                        break;
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -504,16 +492,10 @@ static void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t
     switch (t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) {
         case PA_SUBSCRIPTION_EVENT_SINK_INPUT:
             if ((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE) {
-                QString phononid = s_outputStreamIndexMap.key(index);
-                if (!phononid.isEmpty()) {
-                    if (s_outputStreamIndexMap.contains(phononid)) {
-                        logMessage(QString("Phonon Output Stream %1 is gone at the PA end. Marking it as invalid in our cache as we may reuse it.").arg(phononid));
-                        s_outputStreamIndexMap[phononid] = PA_INVALID_INDEX;
-                    } else {
-                        logMessage(QString("Removing Phonon Output Stream %1 (it's gone!)").arg(phononid));
-                        s_outputStreamIndexMap.remove(phononid);
-                        s_outputStreamMoveQueue.remove(phononid);
-                    }
+                PulseStream *stream = findStreamByPulseIndex(s_outputStreams, index);
+                if (stream) {
+                    logMessage(QString("Phonon Output Stream %1 is gone at the PA end. Marking it as invalid in our cache as we may reuse it.").arg(stream->uuid()));
+                    stream->setIndex(PA_INVALID_INDEX);
                 }
             } else {
                 pa_operation *o;
@@ -527,16 +509,10 @@ static void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t
 
         case PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT:
             if ((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE) {
-                QString phononid = s_captureStreamIndexMap.key(index);
-                if (!phononid.isEmpty()) {
-                    if (s_captureStreamIndexMap.contains(phononid)) {
-                        logMessage(QString("Phonon Capture Stream %1 is gone at the PA end. Marking it as invalid in our cache as we may reuse it.").arg(phononid));
-                        s_captureStreamIndexMap[phononid] = PA_INVALID_INDEX;
-                    } else {
-                        logMessage(QString("Removing Phonon Capture Stream %1 (it's gone!)").arg(phononid));
-                        s_captureStreamIndexMap.remove(phononid);
-                        s_captureStreamMoveQueue.remove(phononid);
-                    }
+              PulseStream *stream = findStreamByPulseIndex(s_captureStreams, index);
+              if (stream) {
+                    logMessage(QString("Phonon Capture Stream %1 is gone at the PA end. Marking it as invalid in our cache as we may reuse it.").arg(stream->uuid()));
+                    stream->setIndex(PA_INVALID_INDEX);
                 }
             } else {
                 pa_operation *o;
@@ -551,64 +527,99 @@ static void subscribe_cb(pa_context *c, pa_subscription_event_type_t t, uint32_t
 }
 
 
-static void ext_device_manager_subscribe_cb(pa_context *c, void *) {
-    Q_ASSERT(c);
-
-    pa_operation *o;
-    PulseUserData *u = new PulseUserData; /** @todo Make some object to receive the info... */
-    if (!(o = pa_ext_device_manager_read(c, ext_device_manager_read_cb, u))) {
-        // We need to deal with failure on first iteration
-        if (s_connectionEventloop) {
-            s_connectionEventloop->exit(0);
-            s_connectionEventloop = NULL;
-        }
-        logMessage(QString("pa_ext_device_manager_read() failed"));
-        return;
+static const char* statename(pa_context_state_t state)
+{
+    switch (state)
+    {
+        case PA_CONTEXT_UNCONNECTED:  return "Unconnected";
+        case PA_CONTEXT_CONNECTING:   return "Connecting";
+        case PA_CONTEXT_AUTHORIZING:  return "Authorizing";
+        case PA_CONTEXT_SETTING_NAME: return "Setting Name";
+        case PA_CONTEXT_READY:        return "Ready";
+        case PA_CONTEXT_FAILED:       return "Failed";
+        case PA_CONTEXT_TERMINATED:   return "Terminated";
     }
-    pa_operation_unref(o);
 
-
-    // Register for the stream changes...
-    pa_context_set_subscribe_callback(c, subscribe_cb, NULL);
-
-    if (!(o = pa_context_subscribe(c, (pa_subscription_mask_t)
-                                   (PA_SUBSCRIPTION_MASK_SINK_INPUT|
-                                    PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT), NULL, NULL))) {
-        logMessage(QString("pa_context_subscribe() failed"));
-        return;
-    }
-    pa_operation_unref(o);
+    static QString unknown;
+    unknown = QString("Unknown state: %0").arg(state);
+    return unknown.toAscii().constData();
 }
 
 static void context_state_callback(pa_context *c, void *)
 {
     Q_ASSERT(c);
 
-    switch (pa_context_get_state(c)) {
-        case PA_CONTEXT_UNCONNECTED:
-        case PA_CONTEXT_CONNECTING:
-        case PA_CONTEXT_AUTHORIZING:
-        case PA_CONTEXT_SETTING_NAME:
-            break;
+    logMessage(QString("context_state_callback %1").arg(statename(pa_context_get_state(c))));
+    pa_context_state_t state = pa_context_get_state(c);
+    if (state == PA_CONTEXT_READY) {
+        // We've connected to PA, so it is active
+        s_pulseActive = true;
 
-        case PA_CONTEXT_READY:
-            // Attempt to load things up
-            ext_device_manager_subscribe_cb(c, NULL);
-            break;
+        // Attempt to load things up
+        pa_operation *o;
 
-        case PA_CONTEXT_FAILED:
-            s_pulseActive = false;
-            if (s_connectionEventloop) {
-                s_connectionEventloop->exit(0);
-                s_connectionEventloop = NULL;
+        // 1. Register for the stream changes (except during probe)
+        if (s_context == c) {
+            pa_context_set_subscribe_callback(c, subscribe_cb, NULL);
+
+            if (!(o = pa_context_subscribe(c, (pa_subscription_mask_t)
+                                              (PA_SUBSCRIPTION_MASK_SINK_INPUT|
+                                               PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT), NULL, NULL))) {
+                logMessage(QString("pa_context_subscribe() failed"));
+                return;
             }
-            break;
+            pa_operation_unref(o);
+        }
 
-        case PA_CONTEXT_TERMINATED:
-        default:
-            s_pulseActive = false;
-            /// @todo Deal with reconnection...
-            break;
+#ifdef HAVE_PULSEAUDIO_DEVICE_MANAGER
+        // 2a. Attempt to initialise Device Manager info (except during probe)
+        if (s_context == c) {
+            pa_ext_device_manager_set_subscribe_cb(c, ext_device_manager_subscribe_cb, NULL);
+            if (!(o = pa_ext_device_manager_subscribe(c, 1, NULL, NULL))) {
+                logMessage(QString("pa_ext_device_manager_subscribe() failed"));
+                return;
+            }
+            pa_operation_unref(o);
+        }
+
+        // 3. Attempt to read info from Device Manager
+        PulseUserData *u = new PulseUserData;
+        if (!(o = pa_ext_device_manager_read(c, ext_device_manager_read_cb, u))) {
+            if (s_context != c) {
+                logMessage(QString("pa_ext_device_manager_read() failed. Attempting to continue without device manager support"));
+                // Only create our gerneric devices during the probe phase.
+                createGenericDevices();
+                // As this is our probe phase, exit immediately
+                pa_context_disconnect(c);
+            }
+            delete u;
+
+            return;
+        }
+        pa_operation_unref(o);
+
+#else
+        // If we know do not have Device Manager support, we just create our dummy devices now
+        if (s_context != c) {
+            // Only create our gerneric devices during the probe phase.
+            createGenericDevices();
+            // As this is our probe phase, exit immediately
+            pa_context_disconnect(c);
+        }
+
+#endif
+    } else if (!PA_CONTEXT_IS_GOOD(state)) {
+        /// @todo Deal with reconnection...
+        //logMessage(QString("Connection to PulseAudio lost: %1").arg(pa_strerror(pa_context_errno(c))));
+
+        // If this is our probe phase, exit our context immediately
+        if (s_context != c)
+            pa_context_disconnect(c);
+        else {
+            pa_context_unref(s_context);
+            s_context = NULL;
+            QTimer::singleShot(50, PulseSupport::getInstance(), SLOT(connectToDaemon()));
+        }
     }
 }
 #endif // HAVE_PULSEAUDIO
@@ -631,7 +642,7 @@ void PulseSupport::shutdown()
 }
 
 PulseSupport::PulseSupport()
- : QObject()
+ : QObject(), mEnabled(false)
 {
 #ifdef HAVE_PULSEAUDIO
     // Initialise our map (is there a better way to do this?)
@@ -647,26 +658,70 @@ PulseSupport::PulseSupport()
 
     // To allow for easy debugging, give an easy way to disable this pulseaudio check
     QString pulseenv = qgetenv("PHONON_PULSEAUDIO_DISABLE");
-    if (pulseenv.toInt())
+    if (pulseenv.toInt()) {
+        logMessage("PulseAudio support disabled: PHONON_PULSEAUDIO_DISABLE is set");
         return;
+    }
 
+    // We require a glib event loop
+    if (QLatin1String(QAbstractEventDispatcher::instance()->metaObject()->className())
+            != "QGuiEventDispatcherGlib") {
+        logMessage("Disabling PulseAudio integration for lack of GLib event loop.");
+        return;
+    }
+
+    // First of all conenct to PA via simple/blocking means and if that succeeds,
+    // use a fully async integrated mainloop method to connect and get proper support.
+    pa_mainloop *p_test_mainloop;
+    if (!(p_test_mainloop = pa_mainloop_new())) {
+        logMessage("PulseAudio support disabled: Unable to create mainloop");
+        return;
+    }
+
+    pa_context *p_test_context;
+    if (!(p_test_context = pa_context_new(pa_mainloop_get_api(p_test_mainloop), "libphonon-probe"))) {
+        logMessage("PulseAudio support disabled: Unable to create context");
+        pa_mainloop_free(p_test_mainloop);
+        return;
+    }
+
+    logMessage("Probing for PulseAudio...");
+    // (cg) Convert to PA_CONTEXT_NOFLAGS when PulseAudio 0.9.19 is required
+    if (pa_context_connect(p_test_context, NULL, static_cast<pa_context_flags_t>(0), NULL) < 0) {
+        logMessage(QString("PulseAudio support disabled: %1").arg(pa_strerror(pa_context_errno(p_test_context))));
+        pa_context_disconnect(p_test_context);
+        pa_context_unref(p_test_context);
+        pa_mainloop_free(p_test_mainloop);
+        return;
+    }
+
+    pa_context_set_state_callback(p_test_context, &context_state_callback, NULL);
+    for (;;) {
+        pa_mainloop_iterate(p_test_mainloop, 1, NULL);
+
+        if (!PA_CONTEXT_IS_GOOD(pa_context_get_state(p_test_context))) {
+            logMessage("PulseAudio probe complete.");
+            break;
+        }
+    }
+    pa_context_disconnect(p_test_context);
+    pa_context_unref(p_test_context);
+    pa_mainloop_free(p_test_mainloop);
+
+    if (!s_pulseActive) {
+        logMessage("PulseAudio support is not available.");
+        return;
+    }
+
+    // If we're still here, PA is available.
+    logMessage("PulseAudio support enabled");
+
+    // Now we connect for real using a proper main loop that we can forget
+    // all about processing.
     s_mainloop = pa_glib_mainloop_new(NULL);
     Q_ASSERT(s_mainloop);
-    pa_mainloop_api *api = pa_glib_mainloop_get_api(s_mainloop);
 
-    // We create a simple event loop to allow the glib loop
-    // to iterate until we've connected or not to the server.
-    s_connectionEventloop = new QEventLoop;
-
-    // XXX I don't want to show up in the client list. All I want to know is the list of sources
-    // and sinks...
-    s_context = pa_context_new(api, "libphonon");
-    // (cg) Convert to PA_CONTEXT_NOFLAGS when PulseAudio 0.9.19 is required
-    if (pa_context_connect(s_context, NULL, static_cast<pa_context_flags_t>(0), 0) >= 0) {
-        pa_context_set_state_callback(s_context, &context_state_callback, s_connectionEventloop);
-        // Now we block until we connect or otherwise...
-        s_connectionEventloop->exec();
-    }
+    connectToDaemon();
 #endif
 }
 
@@ -682,28 +737,34 @@ PulseSupport::~PulseSupport()
         pa_glib_mainloop_free(s_mainloop);
         s_mainloop = NULL;
     }
+#endif
+}
 
-    if (s_connectionEventloop) {
-        delete s_connectionEventloop;
-        s_connectionEventloop = NULL;
-    }
+
+void PulseSupport::connectToDaemon()
+{
+#ifdef HAVE_PULSEAUDIO
+    pa_mainloop_api *api = pa_glib_mainloop_get_api(s_mainloop);
+
+    s_context = pa_context_new(api, "libphonon");
+    if (pa_context_connect(s_context, NULL, PA_CONTEXT_NOFAIL, 0) >= 0)
+        pa_context_set_state_callback(s_context, &context_state_callback, NULL);
 #endif
 }
 
 bool PulseSupport::isActive()
 {
 #ifdef HAVE_PULSEAUDIO
-    return s_pulseActive;
+    //logMessage(QString("Enabled Breakdown: mEnabled: %1, s_pulseActive %2").arg(mEnabled).arg(s_pulseActive));
+    return mEnabled && s_pulseActive;
 #else
     return false;
 #endif
 }
 
-void PulseSupport::disable()
+void PulseSupport::enable(bool enabled)
 {
-#ifdef HAVE_PULSEAUDIO
-    s_pulseActive = false;
-#endif
+    mEnabled = enabled;
 }
 
 QList<int> PulseSupport::objectDescriptionIndexes(ObjectDescriptionType type) const
@@ -821,11 +882,13 @@ static void setDevicePriority(Category category, QStringList list)
     }
     devices[list.size()] = NULL;
 
+#ifdef HAVE_PULSEAUDIO_DEVICE_MANAGER 
     pa_operation *o;
     if (!(o = pa_ext_device_manager_reorder_devices_for_role(s_context, role.toUtf8().constData(), (const char**)devices, NULL, NULL)))
         logMessage(QString("pa_ext_device_manager_reorder_devices_for_role() failed"));
     else
         pa_operation_unref(o);
+#endif
 
     for (i = 0; i < list.size(); ++i)
         pa_xfree(devices[i]);
@@ -869,19 +932,41 @@ void PulseSupport::setCaptureDevicePriorityForCategory(Category category, QList<
 #endif
 }
 
-void PulseSupport::setStreamPropList(Category category, QString streamUuid)
+#ifdef HAVE_PULSEAUDIO
+static PulseStream* register_stream(QMap<QString,PulseStream*> &map, QString streamUuid, Category category)
+{
+    logMessage(QString("Initialising streamindex %1").arg(streamUuid));
+    QString role = s_roleCategoryMap.key(category);
+    if (!role.isEmpty()) {
+        logMessage(QString("Setting role to %1 for streamindex %2").arg(role).arg(streamUuid));
+        setenv("PULSE_PROP_media.role", role.toLatin1().constData(), 1);
+    }
+    setenv("PULSE_PROP_phonon.streamid", streamUuid.toLatin1().constData(), 1);
+    PulseStream *stream = new PulseStream(streamUuid);
+    map[streamUuid] = stream;
+    return stream;
+}
+#endif
+
+PulseStream *PulseSupport::registerOutputStream(QString streamUuid, Category category)
 {
 #ifndef HAVE_PULSEAUDIO
-    Q_UNUSED(category);
     Q_UNUSED(streamUuid);
+    Q_UNUSED(category);
+    return NULL;
 #else
-    QString role = s_roleCategoryMap.key(category);
-    if (role.isEmpty())
-        return;
+    return register_stream(s_outputStreams, streamUuid, category);
+#endif
+}
 
-    logMessage(QString("Setting role to %1 for streamindex %2").arg(role).arg(streamUuid));
-    setenv("PULSE_PROP_media.role", role.toLatin1().constData(), 1);
-    setenv("PULSE_PROP_phonon.streamid", streamUuid.toLatin1().constData(), 1);
+PulseStream *PulseSupport::registerCaptureStream(QString streamUuid, Category category)
+{
+#ifndef HAVE_PULSEAUDIO
+    Q_UNUSED(streamUuid);
+    Q_UNUSED(category);
+    return NULL;
+#else
+    return register_stream(s_captureStreams, streamUuid, category);
 #endif
 }
 
@@ -890,6 +975,19 @@ void PulseSupport::emitObjectDescriptionChanged(ObjectDescriptionType type)
     emit objectDescriptionChanged(type);
 }
 
+bool PulseSupport::setOutputName(QString streamUuid, QString name) {
+#ifndef HAVE_PULSEAUDIO
+    Q_UNUSED(streamUuid);
+    Q_UNUSED(name);
+    return false;
+#else
+    logMessage(QString("Unimplemented: Need to find a way to set either application.name or media.name in SI proplist"));
+    Q_UNUSED(streamUuid);
+    Q_UNUSED(name);
+    return true;
+#endif
+}
+
 bool PulseSupport::setOutputDevice(QString streamUuid, int device) {
 #ifndef HAVE_PULSEAUDIO
     Q_UNUSED(streamUuid);
@@ -906,13 +1004,81 @@ bool PulseSupport::setOutputDevice(QString streamUuid, int device) {
     const QVariant var = s_outputDevices[device].properties["name"];
     logMessage(QString("Attempting to set Output Device to '%1' for Output Stream %2").arg(var.toString()).arg(streamUuid));
 
-    s_outputStreamMoveQueue[streamUuid] = device;
     // Attempt to look up the pulse stream index.
-    if (s_outputStreamIndexMap.contains(streamUuid) && s_outputStreamIndexMap[streamUuid] != PA_INVALID_INDEX) {
+    if (s_outputStreams.contains(streamUuid) && s_outputStreams[streamUuid]->index() != PA_INVALID_INDEX) {
         logMessage(QString("... Found in map. Moving now"));
-        set_output_device(streamUuid);
+
+        uint32_t pulse_device_index = s_outputDevices[device].pulseIndex;
+        uint32_t pulse_stream_index = s_outputStreams[streamUuid]->index();
+
+        logMessage(QString("Moving Pulse Sink Input %1 to '%2' (Pulse Sink %3)").arg(pulse_stream_index).arg(var.toString()).arg(pulse_device_index));
+
+        /// @todo Find a way to move the stream without saving it... We don't want to pollute the stream restore db.
+        pa_operation* o;
+        if (!(o = pa_context_move_sink_input_by_index(s_context, pulse_stream_index, pulse_device_index, NULL, NULL))) {
+            logMessage(QString("pa_context_move_sink_input_by_index() failed"));
+            return false;
+        }
+        pa_operation_unref(o);
     } else {
-        logMessage(QString("... Not found in map. Saving move for when the stream appears"));
+        logMessage(QString("... Not found in map. We will be notified of the device when the stream appears and we can process any moves needed then"));
+    }
+    return true;
+#endif
+}
+
+bool PulseSupport::setOutputVolume(QString streamUuid, qreal volume) {
+#ifndef HAVE_PULSEAUDIO
+    Q_UNUSED(streamUuid);
+    Q_UNUSED(volume);
+    return false;
+#else
+    logMessage(QString("Attempting to set volume to %1 for Output Stream %2").arg(volume).arg(streamUuid));
+
+    // Attempt to look up the pulse stream index.
+    if (s_outputStreams.contains(streamUuid) && s_outputStreams[streamUuid]->index() != PA_INVALID_INDEX) {
+        PulseStream *stream = s_outputStreams[streamUuid];
+
+        uint8_t channels = stream->channels();
+        if (channels < 1) {
+            logMessage("Channel count is less than 1. Cannot set volume.");
+            return false;
+        }
+
+        pa_cvolume vol;
+        pa_cvolume_set(&vol, channels, (volume * PA_VOLUME_NORM));
+
+        logMessage(QString("Found PA index %1. Calling pa_context_set_sink_input_volume()").arg(stream->index()));
+        pa_operation* o;
+        if (!(o = pa_context_set_sink_input_volume(s_context, stream->index(), &vol, NULL, NULL))) {
+            logMessage(QString("pa_context_set_sink_input_volume() failed"));
+            return false;
+        }
+        pa_operation_unref(o);
+    }
+    return true;
+#endif
+}
+
+bool PulseSupport::setOutputMute(QString streamUuid, bool mute) {
+#ifndef HAVE_PULSEAUDIO
+    Q_UNUSED(streamUuid);
+    Q_UNUSED(mute);
+    return false;
+#else
+    logMessage(QString("Attempting to %1 mute for Output Stream %2").arg(mute ? "set" : "unset").arg(streamUuid));
+
+    // Attempt to look up the pulse stream index.
+    if (s_outputStreams.contains(streamUuid) && s_outputStreams[streamUuid]->index() != PA_INVALID_INDEX) {
+        PulseStream *stream = s_outputStreams[streamUuid];
+
+        logMessage(QString("Found PA index %1. Calling pa_context_set_sink_input_mute()").arg(stream->index()));
+        pa_operation* o;
+        if (!(o = pa_context_set_sink_input_mute(s_context, stream->index(), (mute ? 1 : 0), NULL, NULL))) {
+            logMessage(QString("pa_context_set_sink_input_mute() failed"));
+            return false;
+        }
+        pa_operation_unref(o);
     }
     return true;
 #endif
@@ -934,13 +1100,24 @@ bool PulseSupport::setCaptureDevice(QString streamUuid, int device) {
     const QVariant var = s_captureDevices[device].properties["name"];
     logMessage(QString("Attempting to set Capture Device to '%1' for Capture Stream %2").arg(var.toString()).arg(streamUuid));
 
-    s_captureStreamMoveQueue[streamUuid] = device;
     // Attempt to look up the pulse stream index.
-    if (s_captureStreamIndexMap.contains(streamUuid) && s_captureStreamIndexMap[streamUuid] == PA_INVALID_INDEX) {
+    if (s_captureStreams.contains(streamUuid) && s_captureStreams[streamUuid]->index() == PA_INVALID_INDEX) {
         logMessage(QString("... Found in map. Moving now"));
-        set_capture_device(streamUuid);
+
+        uint32_t pulse_device_index = s_captureDevices[device].pulseIndex;
+        uint32_t pulse_stream_index = s_captureStreams[streamUuid]->index();
+
+        logMessage(QString("Moving Pulse Source Output %1 to '%2' (Pulse Sink %3)").arg(pulse_stream_index).arg(var.toString()).arg(pulse_device_index));
+
+        /// @todo Find a way to move the stream without saving it... We don't want to pollute the stream restore db.
+        pa_operation* o;
+        if (!(o = pa_context_move_source_output_by_index(s_context, pulse_stream_index, pulse_device_index, NULL, NULL))) {
+            logMessage(QString("pa_context_move_source_output_by_index() failed"));
+            return false;
+        }
+        pa_operation_unref(o);
     } else {
-        logMessage(QString("... Not found in map. Saving move for when the stream appears"));
+        logMessage(QString("... Not found in map. We will be notified of the device when the stream appears and we can process any moves needed then"));
     }
     return true;
 #endif
@@ -952,10 +1129,15 @@ void PulseSupport::clearStreamCache(QString streamUuid) {
     return;
 #else
     logMessage(QString("Clearing stream cache for stream %1").arg(streamUuid));
-    s_outputStreamIndexMap.remove(streamUuid);
-    s_outputStreamMoveQueue.remove(streamUuid);
-    s_captureStreamIndexMap.remove(streamUuid);
-    s_captureStreamMoveQueue.remove(streamUuid);
+    if (s_outputStreams.contains(streamUuid)) {
+        PulseStream *stream = s_outputStreams[streamUuid];
+        s_outputStreams.remove(streamUuid);
+        delete stream;
+    } else if (s_captureStreams.contains(streamUuid)) {
+        PulseStream *stream = s_captureStreams[streamUuid];
+        s_captureStreams.remove(streamUuid);
+        delete stream;
+    }
 #endif
 }
 
@@ -963,6 +1145,6 @@ void PulseSupport::clearStreamCache(QString streamUuid) {
 
 QT_END_NAMESPACE
 
-#include "moc_pulsesupport_p.cpp"
+#include "moc_pulsesupport.cpp"
 
 // vim: sw=4 ts=4
