@@ -21,7 +21,7 @@
 
 #include "backend.h"
 #include <phonon/experimental/backendinterface.h>
-#include <phonon/pulsesupport_p.h>
+#include <phonon/pulsesupport.h>
 #include "mediaobject.h"
 #include "effect.h"
 #include "events.h"
@@ -41,9 +41,6 @@
 
 #include <QtCore/QByteArray>
 #include <QtCore/QThread>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QSet>
-#include <QtCore/QVariant>
 #include <QtDBus/QDBusConnection>
 #include <QtGui/QApplication>
 #include <QtCore/QSet>
@@ -74,6 +71,7 @@ Backend::Backend(QObject *parent, const QVariantList &)
 {
     // Initialise PulseAudio support
     PulseSupport *pulse = PulseSupport::getInstance();
+    pulse->enable();
     connect(pulse, SIGNAL(objectDescriptionChanged(ObjectDescriptionType)), SLOT(emitObjectDescriptionChanged(ObjectDescriptionType)));
 
     Q_ASSERT(s_instance == 0);
@@ -211,10 +209,6 @@ QStringList Backend::availableMimeTypes() const
 
 QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
 {
-    PulseSupport *pulse = PulseSupport::getInstance();
-    if (pulse->isActive() && (Phonon::AudioOutputDeviceType == type || Phonon::AudioCaptureDeviceType == type))
-        return pulse->objectDescriptionIndexes(type);
-
     QList<int> list;
     switch(type)
     {
@@ -267,10 +261,6 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
 
 QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescriptionType type, int index) const
 {
-    PulseSupport *pulse = PulseSupport::getInstance();
-    if (pulse->isActive() && (Phonon::AudioOutputDeviceType == type || Phonon::AudioCaptureDeviceType == type))
-        return pulse->objectDescriptionProperties(type, index);
-
     //debug() << Q_FUNC_INFO << type << index;
     QHash<QByteArray, QVariant> ret;
     switch (type) {
@@ -591,9 +581,6 @@ QHash<QByteArray, QVariant> Backend::audioOutputProperties(int audioDevice)
 
 QByteArray Backend::audioDriverFor(int audioDevice)
 {
-    if (PulseSupport::getInstance()->isActive())
-        return "pulseaudio";
-
     instance()->checkAudioOutputs();
     const Backend *const that = instance();
     for (int i = 0; i < that->m_audioOutputInfos.size(); ++i) {
@@ -643,7 +630,7 @@ void Backend::checkAudioOutputs()
             }
 
             // We cannot find the output plugin, so let the support class know.
-            pulse->disable();
+            pulse->enable(false);
         }
 
         for (int i = 0; outputPlugins[i]; ++i) {
@@ -686,9 +673,7 @@ void Backend::checkAudioOutputs()
                             "in KDE2 and KDE3. Its use is discouraged.</p></html>"),
                         /*icon name */"audio-backend-arts", outputPlugins[i]);
             } else if (0 == strcmp(outputPlugins[i], "pulseaudio")) {
-                addAudioOutput(nextIndex++, 10, tr("PulseAudio"),
-                        xine_get_audio_driver_plugin_description(m_xine, outputPlugins[i]),
-                        /*icon name */"audio-backend-pulseaudio", outputPlugins[i]);
+                // Ignore this. We deal with it as a special case above.
             } else if (0 == strcmp(outputPlugins[i], "esd")) {
                 addAudioOutput(nextIndex++, 8, tr("Esound (ESD)"),
                         xine_get_audio_driver_plugin_description(m_xine, outputPlugins[i]),
